# AUTO-GENERATED CODE - DO NOT EDIT MANUALLY
# Generated by openapi-python-client from ../../langwatch-saas/langwatch/langwatch/src/app/api/openapiLangWatch.json
# 
# To regenerate: make generate-openapi-client

"""A client library for accessing LangWatch API"""

from .client import AuthenticatedClient, Client

__all__ = (
    "AuthenticatedClient",
    "Client",
)
